@@ -29,14 +29,14 @@ The base class in PluginBase will serve as documentation of available.
 
 import inspect
 import os
-import pkgutil
 import shutil
-from optparse import OptionGroup
+from optparse import OptionConflictError, OptionGroup
 
 import nuitka.plugins.commercial
 import nuitka.plugins.standard
-from nuitka import Options
+from nuitka import Options, OutputDirectories
 from nuitka.__past__ import basestring  # pylint: disable=I0021,redefined-builtin
+from nuitka.__past__ import iter_modules
 from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
 from nuitka.containers.odict import OrderedDict
 from nuitka.containers.oset import OrderedSet
@@ -44,15 +44,17 @@ from nuitka.Errors import NuitkaPluginError
 from nuitka.freezer.IncludedEntryPoints import makeDllEntryPointOld
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
-from nuitka.utils.FileOperations import makePath, relpath
+from nuitka.utils.FileOperations import makePath, putTextFileContents, relpath
 from nuitka.utils.Importing import importFileAsModule
 from nuitka.utils.ModuleNames import ModuleName
 
 from .PluginBase import NuitkaPluginBase, post_modules, pre_modules
 
+# Maps plugin name to plugin instances.
 active_plugins = OrderedDict()
 plugin_name2plugin_classes = {}
 plugin_options = {}
+plugin_datatag2pluginclasses = {}
 plugin_values = {}
 user_plugins = OrderedSet()
 
@@ -89,6 +91,17 @@ def getActivePlugins():
     return active_plugins.values()
 
 
+def getActiveQtPlugin():
+    from .standard.PySidePyQtPlugin import getQtPluginNames
+
+    for plugin_name in getQtPluginNames():
+        if hasActivePlugin(plugin_name):
+            if hasActivePlugin(plugin_name):
+                return plugin_name
+
+    return None
+
+
 def hasActivePlugin(plugin_name):
     """Decide if a plugin is active.
 
@@ -114,6 +127,9 @@ def getPluginClass(plugin_name):
     # First, load plugin classes, to know what we are talking about.
     loadPlugins()
 
+    # Backward compatibility.
+    plugin_name = Options.getPluginNameConsideringRenames(plugin_name)
+
     if plugin_name not in plugin_name2plugin_classes:
         plugins_logger.sysexit("Error, unknown plug-in '%s' referenced." % plugin_name)
 
@@ -121,11 +137,11 @@ def getPluginClass(plugin_name):
 
 
 def _loadPluginClassesFromPath(scan_path):
-    for loader, name, is_pkg in pkgutil.iter_modules(scan_path):
-        if is_pkg:
+    for item in iter_modules(scan_path):
+        if item.ispkg:
             continue
 
-        module_loader = loader.find_module(name)
+        module_loader = item.module_finder.find_module(item.name)
 
         # Ignore bytecode only left overs.
         try:
@@ -137,12 +153,12 @@ def _loadPluginClassesFromPath(scan_path):
             pass
 
         try:
-            plugin_module = module_loader.load_module(name)
+            plugin_module = module_loader.load_module(item.name)
         except Exception:
             if Options.is_nondebug:
                 plugins_logger.warning(
                     "Problem loading plugin %r (%s), ignored. Use --debug to make it visible."
-                    % (name, module_loader.get_filename())
+                    % (item.name, module_loader.get_filename())
                 )
                 continue
 
@@ -188,7 +204,7 @@ def loadStandardPluginClasses():
 
     Notes:
         Scan through the 'standard' and 'commercial' sub-folder of the folder
-        where this script resides. Import each valid Python module (but not
+        where this module resides. Import each valid Python module (but not
         packages) and process it as a plugin.
     Returns:
         None
@@ -198,12 +214,6 @@ def loadStandardPluginClasses():
 
 
 class Plugins(object):
-    @staticmethod
-    def isPluginActive(plugin_name):
-        """ Is a plugin activated. """
-
-        return plugin_name in active_plugins
-
     implicit_imports_cache = {}
 
     @staticmethod
@@ -277,7 +287,8 @@ class Plugins(object):
             )
 
             if decision:
-                imported_module, added_flag = Recursion.recurseTo(
+                imported_module = Recursion.recurseTo(
+                    signal_change=signal_change,
                     module_package=full_name.getPackageName(),
                     module_filename=module_filename,
                     module_relpath=relpath(module_filename),
@@ -287,13 +298,6 @@ class Plugins(object):
 
                 addUsedModule(imported_module)
 
-                if added_flag:
-                    signal_change(
-                        "new_code",
-                        imported_module.getSourceReference(),
-                        "Recursed to module.",
-                    )
-
     @classmethod
     def considerImplicitImports(cls, module, signal_change):
         for plugin in getActivePlugins():
@@ -380,6 +384,21 @@ class Plugins(object):
 
         return result
 
+    @staticmethod
+    def getModuleSpecificDllPaths(module_name):
+        """Provide a list of directories, where DLLs should be searched for this package (or module).
+
+        Args:
+            module_name: name of a package or module, for which the DLL path addition applies.
+
+        """
+        result = OrderedSet()
+        for plugin in getActivePlugins():
+            for dll_path in plugin.getModuleSpecificDllPaths(module_name):
+                result.add(dll_path)
+
+        return result
+
     @staticmethod
     def removeDllDependencies(dll_filename, dll_filenames):
         """Create list of removable shared libraries by scanning through the plugins.
@@ -426,11 +445,145 @@ class Plugins(object):
                 if value:
                     yield plugin, value
 
-    @staticmethod
-    def onModuleDiscovered(module):
+    @classmethod
+    def _createTriggerLoadedModule(cls, module, trigger_name, code, flags):
+        """Create a "trigger" for a module to be imported.
+
+        Notes:
+            The trigger will incorporate the code to be prepended / appended.
+            Called by @onModuleDiscovered.
+
+        Args:
+            module: the module object (serves as dict key)
+            trigger_name: string ("-preload"/"-postload")
+            code: the code string
+
+        Returns
+            trigger_module
+        """
+
+        from nuitka.tree.Building import buildModule
+
+        module_name = ModuleName(module.getFullName() + trigger_name)
+
+        # In debug mode, put the files in the build folder, so they can be looked up easily.
+        if Options.is_debug and "HIDE_SOURCE" not in flags:
+            source_path = os.path.join(
+                OutputDirectories.getSourceDirectoryPath(), module_name + ".py"
+            )
+
+            putTextFileContents(filename=source_path, contents=code)
+
+        try:
+            trigger_module, _added = buildModule(
+                module_filename=os.path.join(
+                    os.path.dirname(module.getCompileTimeFilename()),
+                    module_name.asPath() + ".py",
+                ),
+                module_package=module_name.getPackageName(),
+                source_code=code,
+                is_top=False,
+                is_main=False,
+                is_shlib=False,
+                is_fake=module_name,
+                hide_syntax_error=False,
+            )
+        except SyntaxError:
+            plugins_logger.sysexit(
+                "SyntaxError in plugin provided source code for '%s'." % module_name
+            )
+
+        if trigger_module.getCompilationMode() == "bytecode":
+            trigger_module.setSourceCode(code)
+
+        return trigger_module
+
+    @classmethod
+    def onModuleDiscovered(cls, module):
+        full_name = module.getFullName()
+
+        def _untangleLoadDesc(descs):
+            if descs and inspect.isgenerator(descs):
+                descs = tuple(descs)
+
+            if descs:
+                if type(descs[0]) not in (tuple, list):
+                    descs = [descs]
+
+                for desc in descs:
+                    if len(desc) == 2:
+                        code, reason = desc
+                        flags = ()
+                    else:
+                        code, reason, flags = desc
+                        if type(flags) is str:
+                            flags = (flags,)
+
+                    yield plugin, code, reason, flags
+
+        preload_descs = []
+        postload_descs = []
+
         for plugin in getActivePlugins():
             plugin.onModuleDiscovered(module)
 
+            preload_descs.extend(
+                _untangleLoadDesc(descs=plugin.createPreModuleLoadCode(module))
+            )
+            postload_descs.extend(
+                _untangleLoadDesc(descs=plugin.createPostModuleLoadCode(module))
+            )
+
+        if preload_descs:
+            total_code = []
+            total_flags = OrderedSet()
+
+            for plugin, pre_code, reason, flags in preload_descs:
+                if pre_code:
+                    plugin.info(
+                        "Injecting pre-module load code for module '%s':" % full_name
+                    )
+                    for line in reason.split("\n"):
+                        plugin.info("    " + line)
+
+                    total_code.append(pre_code)
+                    total_flags.update(flags)
+
+            if total_code:
+                assert full_name not in pre_modules
+
+                pre_modules[full_name] = cls._createTriggerLoadedModule(
+                    module=module,
+                    trigger_name="-preLoad",
+                    code="\n\n".join(total_code),
+                    flags=total_flags,
+                )
+
+        if postload_descs:
+            total_code = []
+            total_flags = OrderedSet()
+
+            for plugin, post_code, reason, flags in postload_descs:
+                if post_code:
+                    plugin.info(
+                        "Injecting post-module load code for module '%s':" % full_name
+                    )
+                    for line in reason.split("\n"):
+                        plugin.info("    " + line)
+
+                    total_code.append(post_code)
+                    total_flags.update(flags)
+
+            if total_code:
+                assert full_name not in post_modules
+
+                post_modules[full_name] = cls._createTriggerLoadedModule(
+                    module=module,
+                    trigger_name="-postLoad",
+                    code="\n\n".join(total_code),
+                    flags=total_flags,
+                )
+
     @staticmethod
     def onModuleSourceCode(module_name, source_code):
         assert type(module_name) is ModuleName
@@ -481,6 +634,11 @@ class Plugins(object):
             if must_recurse is None:
                 continue
 
+            if type(must_recurse) is not tuple and must_recurse not in (True, False):
+                plugin.sysexit(
+                    "Error, onModuleEncounter code failed to return a None or tuple(bool, reason) result."
+                )
+
             if result is not None:
                 # false alarm, pylint: disable=unsubscriptable-object
                 assert result[0] == must_recurse[0]
@@ -651,6 +809,17 @@ class Plugins(object):
 
         return name
 
+    @classmethod
+    def onFunctionAssignmentParsed(cls, function_body, assign_node):
+        module_name = function_body.getParentModule().getFullName()
+
+        for plugin in getActivePlugins():
+            plugin.onFunctionAssignmentParsed(
+                module_name=module_name,
+                function_body=function_body,
+                assign_node=assign_node,
+            )
+
 
 def listPlugins():
     """Print available standard plugins."""
@@ -785,6 +954,9 @@ def activatePlugins():
         plugin_name2plugin_classes.items()
     ):
         if plugin_name in Options.getPluginsEnabled():
+            if plugin_class.isAlwaysEnabled():
+                plugin_class.warning("Plugin is defined as always enabled.")
+
             if plugin_class.isRelevant():
                 _addActivePlugin(plugin_class, args=True)
             else:
@@ -816,19 +988,48 @@ def lateActivatePlugin(plugin_name, option_values):
     _addActivePlugin(getPluginClass(plugin_name), args=True, force=True)
 
 
-def _addPluginCommandLineOptions(parser, plugin_class):
-    if plugin_class.plugin_name not in plugin_options:
-        option_group = OptionGroup(parser, "Plugin %s" % plugin_class.plugin_name)
-        plugin_class.addPluginCommandLineOptions(option_group)
+def _addPluginCommandLineOptions(parser, plugin_class, data_files_tags):
+    plugin_name = plugin_class.plugin_name
+
+    if plugin_name not in plugin_options:
+        option_group = OptionGroup(parser, "Plugin %s" % plugin_name)
+        try:
+            plugin_class.addPluginCommandLineOptions(option_group)
+        except OptionConflictError as e:
+            for other_plugin_name, other_plugin_option_list in plugin_options.items():
+                for other_plugin_option in other_plugin_option_list:
+                    # no public interface for that, pylint: disable=protected-access
+                    if (
+                        e.option_id in other_plugin_option._long_opts
+                        or other_plugin_option._short_opts
+                    ):
+                        plugins_logger.sysexit(
+                            "Plugin '%s' failed to add options due to conflict with '%s' from plugin '%s."
+                            % (plugin_name, e.option_id, other_plugin_name)
+                        )
 
         if option_group.option_list:
             parser.add_option_group(option_group)
-            plugin_options[plugin_class.plugin_name] = option_group.option_list
+            plugin_options[plugin_name] = option_group.option_list
         else:
-            plugin_options[plugin_class.plugin_name] = ()
+            plugin_options[plugin_name] = ()
+
+        plugin_data_files_tags = plugin_class.getTagDataFileTagOptions()
+
+        if plugin_data_files_tags:
+            for tag_name, tag_desc in plugin_data_files_tags:
+                if tag_name in (tag for tag, _desc in data_files_tags):
+                    plugins_logger.sysexit(
+                        "Plugin '%s' provides data files tag handling '%s' already provided."
+                        % (plugin_name, tag_name)
+                    )
+
+                data_files_tags.append((tag_name, tag_desc))
+
+                plugin_datatag2pluginclasses[tag_name] = plugin_class
 
 
-def addPluginCommandLineOptions(parser, plugin_names):
+def addPluginCommandLineOptions(parser, plugin_names, data_files_tags):
     """Add option group for the plugin to the parser.
 
     Notes:
@@ -842,12 +1043,16 @@ def addPluginCommandLineOptions(parser, plugin_names):
     """
     for plugin_name in plugin_names:
         plugin_class = getPluginClass(plugin_name)
-        _addPluginCommandLineOptions(parser, plugin_class)
+        _addPluginCommandLineOptions(
+            parser=parser, plugin_class=plugin_class, data_files_tags=data_files_tags
+        )
 
 
-def addUserPluginCommandLineOptions(parser, filename):
+def addUserPluginCommandLineOptions(parser, filename, data_files_tags):
     plugin_class = loadUserPlugin(filename)
-    _addPluginCommandLineOptions(parser, plugin_class)
+    _addPluginCommandLineOptions(
+        parser=parser, plugin_class=plugin_class, data_files_tags=data_files_tags
+    )
 
     user_plugins.add(plugin_class)
 
