@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,6 +22,7 @@ Here the small things that fit nowhere else and don't deserve their own module.
 """
 
 import os
+import platform
 import sys
 
 from nuitka.PythonVersions import python_version
@@ -42,17 +43,62 @@ def getOS():
         assert False, os.name
 
 
-def isWin32Windows():
-    """ The Win32 variants of Python does have win32 only, not posix.
+_linux_distribution = None
 
+
+def getLinuxDistribution():
+    """Name of the Linux distribution.
+
+    We should usually avoid this, and rather test for the feature,
+    but in some cases it's hard to manage that.
     """
+    # singleton, pylint: disable=global-statement
+    global _linux_distribution
+
+    if getOS() != "Linux":
+        return None
+
+    if _linux_distribution is None:
+        # pylint: disable=I0021,deprecated-method,no-member
+        try:
+            result = platform.dist()[0].title()
+        except AttributeError:
+            from .Execution import check_output
+
+            try:
+                result = check_output(["lsb_release", "-i", "-s"], shell=False).title()
+
+                if str is not bytes:
+                    result = result.decode("utf8")
+            except FileNotFoundError:
+                from .FileOperations import getFileContentByLine
+
+                for line in getFileContentByLine("/etc/os-release"):
+                    if line.startswith("ID="):
+                        result = line[3:]
+                        break
+                else:
+                    from nuitka.Tracing import general
+
+                    general.sysexit("Error, cannot detect Linux distribution.")
+
+        _linux_distribution = result.title()
+
+    return _linux_distribution
+
+
+def isDebianBasedLinux():
+    # TODO: What is with Mint, maybe others, this list should be expanded potentially.
+    return getLinuxDistribution() in ("Debian", "Ubuntu")
+
+
+def isWin32Windows():
+    """The Win32 variants of Python does have win32 only, not posix."""
     return os.name == "nt"
 
 
 def isPosixWindows():
-    """ The MSYS2 variant of Python does have posix only, not Win32.
-
-    """
+    """The MSYS2 variant of Python does have posix only, not Win32."""
     return os.name == "posix" and getOS() == "Windows"
 
 
@@ -82,32 +128,6 @@ def getArchitecture():
         return os.uname()[4]
 
 
-def getSharedLibrarySuffix():
-    if python_version < 300:
-        import imp
-
-        result = None
-
-        for suffix, _mode, module_type in imp.get_suffixes():
-            if module_type != imp.C_EXTENSION:
-                continue
-
-            if result is None or len(suffix) < len(result):
-                result = suffix
-
-        return result
-    else:
-        import importlib.machinery  # pylint: disable=I0021,import-error,no-name-in-module
-
-        result = None
-
-        for suffix in importlib.machinery.EXTENSION_SUFFIXES:
-            if result is None or len(suffix) < len(result):
-                result = suffix
-
-        return result
-
-
 def getCoreCount():
     cpu_count = 0
 
@@ -128,12 +148,12 @@ def getCoreCount():
 
 
 def encodeNonAscii(var_name):
-    """ Encode variable name that is potentially not ASCII to ASCII only.
+    """Encode variable name that is potentially not ASCII to ASCII only.
 
-        For Python3, unicode identifiers can be used, but these are not
-        possible in C, so we need to replace them.
+    For Python3, unicode identifiers can be used, but these are not
+    possible in C, so we need to replace them.
     """
-    if python_version < 300:
+    if python_version < 0x300:
         return var_name
     else:
         # Using a escaping here, because that makes it safe in terms of not
@@ -146,17 +166,15 @@ def encodeNonAscii(var_name):
         return var_name.replace("&#", "$$").replace(";", "")
 
 
-def isExecutableCommand(command):
-    path = os.environ["PATH"]
+def hasOnefileSupportedOS():
+    return getOS() in ("Linux", "Windows", "Darwin", "FreeBSD")
 
-    suffixes = (".exe",) if getOS() == "Windows" else ("",)
 
-    for part in path.split(os.pathsep):
-        if not part:
-            continue
+def getUserName():
+    """Return the user name.
 
-        for suffix in suffixes:
-            if os.path.isfile(os.path.join(part, command + suffix)):
-                return True
+    Notes: Currently doesn't work on Windows.
+    """
+    import pwd  # pylint: disable=I0021,import-error
 
-    return False
+    return pwd.getpwuid(os.getuid())[0]
