@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -21,4 +21,12 @@ import numpy
 
 # nuitka-skip-unless-imports: numpy
 
-numpy.test()
+# nuitka-project: --standalone
+# nuitka-project: --enable-plugin=numpy
+
+# Make sure, the usual bad ones are not included with anti-bloat.
+
+# nuitka-project: --enable-plugin=anti-bloat
+# nuitka-project: --noinclude-setuptools-mode=error
+# nuitka-project: --noinclude-pytest-mode=error
+# nuitka-project: --noinclude-custom-mode=numpy.distutils:error
